-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  Serializer
-- File Name:    Serializer.vhd
-- File Path:    E:\Projects\BlockRamLeftOvers\Serializer\Vhdl\
-- Project :	BlockRam Serializer and storage
--
-- Purpose: 	Demo that a BlockRAM can be used as a serializer.
--				This is a simple version of the serializer. No pattern lookup circuit is
--				involved. When 32-bits are shifted into the RAM, the next address location 
--				is selected and the process continues.
--
--				Simulation files for this small design are included in the
--				...../SimScript directory.
--					SerializerComp.do, SerializerStimu.do and SerializerWave.do.
--
-- Disclaimer :
--	These designs are provided "as is" with no warranty whatsoever, and xilinx 
--	specifically disclaims any implied warranties of merchantability, fitness 
--	for a particular purpose, or against infringement.
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
--
-- Tools: XST 5.2sp3, Synplicity 7.2.1, ModelSim SE5.7a
--
-- Revision History:       Created:       02/06/03
--                         Last opened:   maandag,jun 02 03
--
--
----------------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- synopsys translate_off
library UNISIM;
use UNISIM.all;
-- synopsys translate_on

entity Serializer is
	generic (
		NumberOfDataBits : integer := 32;
		NumberOfAddrBits : integer := 9
	);
    port (
		Reset		: in std_logic;
		ClockA		: in std_logic;
		EnableA		: in std_logic;
		DataInA		: in std_logic;
		DataOutA	: out std_logic_vector (NumberOfDataBits-1 downto 0);
		
		ClockB		: in std_logic;
		EnableB		: in std_logic;
		AddressB	: in std_logic_vector (NumberOfAddrBits-1 downto 0);
		DataOutB	: out std_logic_vector (NumberOfDataBits-1 downto 0)
    );		
		-- Attributes for Synplicity, not to put the registers in the IOB FFs.
		-- Don't forget to turn it off in the ISE software !
		attribute syn_useioff : boolean;
		attribute syn_useioff of DataInA : signal is false;
		attribute syn_useioff of DataOutA : signal is false;
		attribute syn_useioff of AddressB : signal is false;
		attribute syn_useioff of DataOutB : signal is false;
end  Serializer ;

architecture Serializer_arch of Serializer  is
----------------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
----------------------------------------------------------------------------------------------------------------------------------------
-- Clock stuff
--
component dcm 
	port (
		CLKIN   	: in  std_logic ;
		CLKFB   	: in  std_logic ;
		DSSEN 		: in  std_logic ;
		PSINCDEC	: in  std_logic ;
		PSEN 		: in  std_logic ;
		PSCLK 		: in  std_logic ;
		RST     	: in  std_logic ;
		CLK0    	: out std_logic ;
		CLK90   	: out std_logic ;
		CLK180  	: out std_logic ;
		CLK270  	: out std_logic ;
		CLK2X   	: out std_logic ;
		CLK2X180	: out std_logic ;
		CLKDV   	: out std_logic ;
		CLKFX   	: out std_logic ;
		CLKFX180	: out std_logic ;
		LOCKED  	: out std_logic ;
		PSDONE  	: out std_logic ;
		STATUS  	: out std_logic_vector(7 downto 0)
	); 
end component;

component IBUFG 
	port (
		I :	in std_logic;
		O : out std_logic
	); 
end component;

component BUFG
	port (
		I : in std_logic;
		O : out std_logic
	);
end component;
--
-- End Clock Stuff
--------------------------------------------------------------------------------------------
-- Components instantiated for the sample design
--
component RAMB16_S36_S36
	port (
		WEA		: in std_logic;
		ENA		: in std_logic;
		SSRA	: in std_logic;
		CLKA	: in std_logic;
		ADDRA	: in std_logic_vector (8 downto 0);
		DIA		: in std_logic_vector (31 downto 0);
		DIPA	: in std_logic_vector (3 downto 0);
		DOA		: out std_logic_vector (31 downto 0);
		DOPA	: out std_logic_vector (3 downto 0);
		WEB		: in std_logic;
		ENB		: in std_logic;
		SSRB	: in std_logic;
		CLKB	: in std_logic;
		ADDRB	: in std_logic_vector (8 downto 0);
		DIB		: in std_logic_vector (31 downto 0);
		DIPB	: in std_logic_vector (3 downto 0);
		DOB		: out std_logic_vector (31 downto 0);
		DOPB	: out std_logic_vector (3 downto 0)
	);
end component;

component SRL16E
	port (
		D	: in std_logic;
		CE	: in std_logic;
		CLK	: in std_logic;
		A0	: in std_logic;
		A1	: in std_logic;
		A2	: in std_logic;
		A3	: in std_logic;
		Q	: out std_logic
	);
end component;
----------------------------------------------------------------------------------------------------------------------------------------
-- Signal Declarations
----------------------------------------------------------------------------------------------------------------------------------------
signal Low : std_logic;
signal High : std_logic;

signal IntClockA		: std_logic;
signal ClkToDcm			: std_logic;
signal ClkToBUfg		: std_logic;
signal ClkToFeedBack	: std_logic;

signal IntDataInA		: std_logic;
signal IntDatInA		: std_logic_vector (NumberOfDataBits-1 downto 0);
signal IntDatOutA		: std_logic_vector (NumberOfDataBits-1 downto 0);
signal IntAddrB			: std_logic_vector (NumberOfAddrBits-1 downto 0);
signal IntDataOutB		: std_logic_vector (NumberOfDataBits-1 downto 0);
--
-- These ports, signals are not used but declared for synthesis purposes.
-- Most Of them are tied to GND.
signal IntDatInParA		: std_logic_vector ((NumberOfDataBits/8)-1 downto 0);
--signal IntDatOutParA	: std_logic_vector ((NumberOfDataBits/8)-1 downto 0);
signal IntDatInB		: std_logic_vector (NumberOfDataBits-1 downto 0);
signal IntDatInParB		: std_logic_vector ((NumberOfDataBits/8)-1 downto 0);
--signal IntDatOutParB	: std_logic_vector ((NumberOfDataBits/8)-1 downto 0);
--
signal IntAddrCnt		: std_logic_vector (NumberOfAddrBits-1 downto 0);
--
signal IntBitCntInit	: std_logic;
signal IntBitCntIn		: std_logic;
signal IntBitCntBitCnt	: std_logic;
signal IntBitCntOut		: std_logic;
signal IntBitCnt		: std_logic_vector (4 downto 0);
signal IntBitCntOutTc	: std_logic;
--
attribute DLL_FREQUENCY_MODE : string;
attribute MAXDELAY : string;
attribute syn_keep : integer;

attribute DLL_FREQUENCY_MODE of TheDcm: label is "HIGH";
attribute MAXDELAY of IntBitCntBitCnt: signal is "500ps";
attribute MAXDELAY of ClkToDcm: signal is "1000ps";
attribute MAXDELAY of IntDatOutA: signal is "2000ps";

attribute syn_keep of IntBitCntBitCnt: signal is 1;
attribute syn_keep of ClkToDcm: signal is 1;
attribute syn_keep of IntDatOutA: signal is 1;
----------------------------------------------------------------------------------------------------------------------------------------
--
begin
--------------------------------------------------------------------------------
-- Clock Stuff of test design.
--------------------------------------------------------------------------------
TheDiffClkIn : IBUFG
	port map (
		I	=> ClockA,
		O	=> ClkToDcm
	);
TheDcm : DCM	
	port map (
		CLKIN   	=> ClkToDcm,
		CLKFB   	=> ClkToFeedBack,
		DSSEN 		=> low,
		PSINCDEC	=> low,
		PSEN 		=> low,
		PSCLK 		=> low,
		RST     	=> Reset,
		CLK0    	=> ClkToBufg,
		CLK90   	=> open,
		CLK180  	=> open,
		CLK270  	=> open,
		CLK2X   	=> open,
		CLK2X180	=> open,
		CLKDV   	=> open,
		CLKFX   	=> open,
		CLKFX180	=> open,
		LOCKED  	=> open,
		PSDONE  	=> open,
		STATUS  	=> open
	);
TheBufG : BUFG
	port map (
		I	=> ClkToBufg,
		O	=> ClkToFeedBack
	);

IntClockA <= ClkToFeedBack;
--
--------------------------------------------------------------------------------
-- The sample design
--------------------------------------------------------------------------------
-- Fixed design things 
--
High <= '1';
Low <= '0';
IntDatInParA	<= (others => '0'); -- because these busses are not used, they are tied to GND.
IntDatInB 		<= (others => '0'); -- else the synthesis tools complain and error out.
IntDatInParB	<= (others => '0'); --
--
-- Set of input and output registers.
-- Only here for ease of use showng the design.
--
TheIoRegsA : process (IntClockA, Reset, EnableA)
begin
	if (Reset = '1' ) then
		IntDataInA <= '0';
		DataOutA <= (others => '0');
	elsif (IntClockA'event and IntClockA = '1') then
		if (EnableA = '1') then
			IntDataInA <= DataInA;
			DataOutA <= IntDatOutA;
		end if;
	end if;
end process;
TheIoRegsB : process (ClockB, Reset)
begin
	if (Reset = '1' ) then
	DataOutB <= (others => '0');
	IntAddrB <= (others => '0');
	elsif (ClockB'event and ClockB = '1') then
		DataOutB <= IntDataOutB;
		IntAddrB <= AddressB;
	end if;
end process;
--

TheBram : RAMB16_S36_S36
	port map (
		WEA		=> High,   -- Write always
		ENA		=> EnableA,
		SSRA	=> Reset,
		CLKA	=> IntClockA,
		ADDRA	=> IntAddrCnt,
		DIA		=> IntDatInA,
		DIPA	=> IntDatInParA,
		DOA		=> IntDatOutA,
		DOPA	=> open, --IntDatOutParA,
		--
		WEB		=> Low,		-- read always
		ENB		=> EnableB,
		SSRB	=> Reset,
		CLKB	=> ClockB,
		ADDRB	=> IntAddrB,
		DIB		=> IntDatInB,
		DIPB	=> IntDatInParB,
		DOB		=> IntDataOutB,
		DOPB	=> open
	);
--
IntDatInA <= IntDatOutA(NumberOfDataBits-2 downto 0) & IntDataInA;
-- The data feedback loop must be faster than the clock period.

-- Address counter.
-- When the BRAM is only used a serializer this address counter is not needed.
-- Fix the address to whatever value and the serializer is ready.
-- This counter runs at a rate x-time lower than the BlockRAM, where x is the width of
-- the blockRAM data bus.
--
-- This address counter let the BlockRAm act as a serializer with attached FIFO memory.
-- When a data fram is shift into the BlockRAM the address is incremented, and shifting 
-- can start again.
-- Port B of the BlockRam can be used to empty the memory.
--
TheAddrCnt : process (IntClockA, Reset, EnableA, IntBitCntOut)
	begin
		if (reset = '1') then
			IntAddrCnt <= (others => '0');
		elsif (IntClockA'event and IntClockA = '0') then
			if (EnableA = '1' and IntBitCntOut = '1') then
				IntAddrCnt <= IntAddrCnt + "01";
			end if;
		end if;
	end process;
--
-- A circuit indicating the end of the shift register is in most cases needed.
-- An example why it can be used is; increment the BlockRAM address counter.
-- This circuit can be constructed out of as SRL configured LUTs. The circuit then
-- acts as a with-the-data-running-bit.
--
-- This circuit needs to run as fast as the BlockRAM, it's a bit indicator.
-- When done with SRL registers the circuit is very small, but faerly slow.
--
-- This is the circuit done in SRL.   
-- If this is wanted, the speed of the serializer is not that high, uncomment this
-- syntax and comment the counter syntax.
--
--TheInitFf : process (IntClockA, Reset, EnableA)
--	begin
--		if (Reset = '1') then
--			IntBitCntInit <= '1';
--		elsif (IntClockA'event and IntClockA = '1') then
--			if (EnableA = '1') then
--				IntBitCntInit <= '0';
--			end if;
--		end if;
--	end process;
--
--IntBitCntIn <= IntBitCntInit or IntBitCntOut;
--
--TheBitCntOne : SRL16E
--	port map (
--		D	=> IntBitCntIn,
--		CE	=> EnableA,
--		CLK	=> IntClockA,
--		A0	=> High,
--		A1	=> High,
--		A2	=> High,
--		A3	=> High,
--		Q	=> IntBitCntBitCnt
--	);
--TheBitCntTwo : SRL16E
--	port map (
--		D	=> IntBitCntBitCnt,
--		CE	=> EnableA,
--		CLK	=> IntClockA,
--		A0	=> High,
--		A1	=> High,
--		A2	=> High,
--		A3	=> High,
--		Q	=> IntBitCntOut
--	);
-- Uncomment till here when the SRL approach is needed
--
-- When the circuit need to be fast, better is do construct is as a counter.
-- For a 32-bit shift register a 5-bit counter is needed.
--
TheBitCnt : process (IntClockA, Reset, EnableA)
begin
	if (Reset = '1') then
		IntBitCnt <= (others => '0');
		IntBitCntOut <= '0';
	elsif (IntClockA'event and IntClockA = '1') then
		if (EnableA = '1') then
			IntBitCnt <= IntBitCnt + "01";
			IntBitCntOut <= IntBitCntOutTc;
		end if;
	end if;
end process;
IntBitCntOutTc <= '1' when (IntBitCnt = "11110") else '0';
--
--
end  Serializer_arch ;